package com.icontrols.kakao.controller;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.SERVERINFO;
import com.icontrols.kakao.common.TYPE;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.common.utils.JWTUtils2;
import com.icontrols.kakao.entity.AccessLog;
import com.icontrols.kakao.repo.AccessLogRepository;
import com.icontrols.kakao.service.KakaoExtensionService;

@RestController
public class KakaoDanziController {
	
	private static final Logger logger = LoggerFactory.getLogger(KakaoDanziController.class);
	
	@Autowired
	KakaoExtensionService kakaoService;
	
	@Autowired
	AccessLogRepository accessLogRepository;
	
	/**
	 * 중계 서버에서 오는 요청을         분기 처리를 진행한다.
	 * @param header
	 * @return
	 */
	@RequestMapping(value = "/kakao/danzi", method = RequestMethod.POST, consumes = "application/json")
	public @ResponseBody Map<String, Object> getRequest(
														@RequestHeader HttpHeaders header,
														@RequestBody Map<String, Object> params,
														HttpServletResponse response, 
														HttpServletRequest request
														) throws Exception{
		
		
		try {
			String requestId = (String)params.get("id");
			String requestType = (String) params.get("type");
			String authorization = (String) params.get("authorization");
			String body = params.containsKey("body")?params.get("body").toString():"";
			
			if(body.length() > 255) body = body.substring(0, 255);
			
			AccessLog log = new AccessLog(requestId, authorization, requestType, body);
			accessLogRepository.save(log);
		}catch(Exception e) {
			logger.error("log 저장 실패 " + e.getMessage());
			e.printStackTrace();
		}
		
		logger.debug("KAKAO/DANZI %s"+ params.toString());
		
		// 인증은 여기서 해야하나??  Interceptor에서 해야하나?? 일단 그부분은 고민을 해보자.
		if(!params.containsKey(KEY_INFO.HTTP_KEY_AUTHORIZATION)) {
			logger.error("[ERROR] - 인증토큰 존재하지 않음");
			
			params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_TOKEN));
			return params;
		}
		
		int token_vali = JWTUtils2.validateToken((String)params.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
		if(token_vali == KAKAO_RESULT.FAIL_EXPIREDTOKEN) {
			logger.error("[ERROR] - 인 증 토 큰 만 료");
			logger.error("[ERROR] - " + (String)params.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
			
			params.remove(KEY_INFO.HTTP_KEY_AUTHORIZATION);
			if(params.containsKey(KEY_INFO.HTTP_KEY_BODY)) params.remove(KEY_INFO.HTTP_KEY_BODY);
			
			params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_EXPIREDTOKEN));
			
			return params;
		}else if(token_vali == KAKAO_RESULT.FAIL_INVALID_TOKEN) {
			logger.error("[ERROR] - 유 효 하 지 않 는 토 큰");
			logger.error("[ERROR] - " + (String)params.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
			
			params.remove(KEY_INFO.HTTP_KEY_AUTHORIZATION);
			if(params.containsKey(KEY_INFO.HTTP_KEY_BODY)) params.remove(KEY_INFO.HTTP_KEY_BODY);
			
			params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_TOKEN));
			return params;
		}
		
		// 받은 요청에 대해 반환 해주는 결과 JSON 객체
		Map<String, Object> retMap = null;
		
		try {
			//Request Parameter에서  type값을 통해 분기를 태운다. 
			String reqType = (params.containsKey(KEY_INFO.HTTP_KEY_TYPE))?(String) params.get(KEY_INFO.HTTP_KEY_TYPE):"";
			logger.info("[REQUEST INBOUD] reqType = " + reqType);
			
			if(reqType.equals(TYPE.REQ_TYPE.GRANTAUTH.getTypeName())) {
				// grantauth 인 경우에만 헤더의 인증 필드를 검증한다.
				
				if(!header.containsKey(SERVERINFO.HEADER_AUTH)) {
					params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_CLIENT));
					return params;
				}else {
					String authorization =  request.getHeader(SERVERINFO.HEADER_AUTH);
					logger.info(authorization);
					int validation_auth = CommonUtil.validateHeaderAuth(authorization);
					if(validation_auth != KAKAO_RESULT.SUCCESS) {
						params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_CLIENT));
						return params;
					}
				}
				
				// 카카오 홈과 유저의 인증 완료시 카카오에서 전송해주는 API TYPE
				// 건내받은 토큰 정보를 DB에 저장해야 한다.
				retMap = kakaoService.request_grantAuthorization(params);
			} else if(reqType.equals(TYPE.REQ_TYPE.DISCOVER.getTypeName())) {
				// 월패드와 연결된 기기목록들을 요청하는 API TYPE
				retMap = kakaoService.request_discover(params);
			} else if(reqType.equals(TYPE.REQ_TYPE.QUERYSTATE.getTypeName())) {
				// 월패드와 연결된 기기중 endpointIds에 포함된 기기의 상태값을 확인한다.
				retMap = kakaoService.request_queryState(params);
			} else if(reqType.equals(TYPE.REQ_TYPE.UPDATESTATE.getTypeName())) {
				logger.info("Update_state --> params \r\n" + params.toString());
				// 월패드와 연결된 기기의 상태를 번경한다. 
				retMap = kakaoService.request_updateState(params);
			}else {
				// 정의되지않은 type이 들어왔을경우.
				params.remove(KEY_INFO.HTTP_KEY_BODY);
				params.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INVALID_REQ));
				retMap = params;
			}
		}catch(Exception e) {
			logger.error("[Unknown ERROR]");
			e.printStackTrace();
			retMap = KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL);
		}finally {
			retMap.remove(KEY_INFO.HTTP_KEY_AUTHORIZATION);
		}

		return retMap;
	}
}
